/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is about:memory.
 *
 * The Initial Developer of the Original Code is the Mozilla Foundation.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Vladimir Vukicevic <vladimir@pobox.com>
 *   Nicholas Nethercote <nnethercote@mozilla.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

.mrValue {
  font-weight: bold;
  color: #400;
}

.mrPerc {
}

.mrName {
  color: #004;
}

.hasDesc:hover {
  text-decoration: underline;
}

.mrStar {
  color: #604;
}

.treeLine {
  color: #888;
}

.option {
  font-size: 80%;
  -moz-user-select: none;  /* no need to include this when cutting+pasting */
}

.legend {
  font-size: 80%;
  -moz-user-select: none;  /* no need to include this when cutting+pasting */
}

body.verbose {
  /* override setting in about.css */
  max-width: 100% !important;
}

h2.tree {
  cursor: pointer;
  background: #ddd;
  padding-left: .1em;
}

body.non-verbose pre.tree {
  overflow-x: hidden;
  text-overflow: ellipsis;
}

pre.collapsed {
  display: none;
}
