/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt <hewitt@netscape.com> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"

interface nsIDocShell;
interface nsIAutoCompletePopup;
interface nsIDOMHTMLInputElement;

/*
 * nsIFormFillController is an interface for controlling form fill behavior
 * on HTML documents.  Any number of docShells can be controller concurrently.
 * While a docShell is attached, all HTML documents that are loaded within it
 * will have a focus listener attached that will listen for when a text input
 * is focused.  When this happens, the input will be bound to the
 * global nsIAutoCompleteController service.
 */

[scriptable, uuid(07f0a0dc-f6e9-4cdd-a55f-56d770523a4c)]
interface nsIFormFillController : nsISupports
{
  /*
   * Start controlling form fill behavior for the given browser
   *
   * @param docShell - The docShell to attach to
   * @param popup - The popup to show when autocomplete results are available
   */
  void attachToBrowser(in nsIDocShell docShell, in nsIAutoCompletePopup popup);

  /*
   * Stop controlling form fill behavior for the given browser
   *
   * @param docShell - The docShell to detach from
   */
  void detachFromBrowser(in nsIDocShell docShell);

  /*
   * Mark the specified <input> element as being managed by password manager.
   * Autocomplete requests will be handed off to the password manager, and will
   * not be stored in form history.
   *
   * @param aInput - The HTML <input> element to tag
   */
  void markAsLoginManagerField(in nsIDOMHTMLInputElement aInput);
};
